@@ -22,11 +22,10 @@
 class RooExtendedTerm : public RooAbsPdf {
 public:
 
-  RooExtendedTerm() ;
+  RooExtendedTerm() = default;
   RooExtendedTerm(const char *name, const char *title, const RooAbsReal& n) ;
   RooExtendedTerm(const RooExtendedTerm& other, const char* name=nullptr) ;
-  TObject* clone(const char* newname) const override { return new RooExtendedTerm(*this,newname) ; }
-  ~RooExtendedTerm() override ;
+  TObject* clone(const char* newname=nullptr) const override { return new RooExtendedTerm(*this,newname) ; }
 
   double evaluate() const override { return 1. ; }
 
@@ -34,6 +33,8 @@ public:
   /// Return number of expected events, in other words the value of the associated n parameter.
   double expectedEvents(const RooArgSet* nset) const override ;
 
+  std::unique_ptr<RooAbsReal> createExpectedEventsFunc(const RooArgSet* nset) const override;
+
 protected:
 
   RooRealProxy _n ;          ///< Number of expected events
