@@ -31,6 +31,7 @@
 #include "Rtypes.h"
 #include "TError.h"
 
+#include <cassert>
 
 template<class Element> class TMatrixTSym;
 template<class Element> class TMatrixTSparse;
@@ -107,6 +108,8 @@ public:
    void MultT(const TMatrixTSym<Element> &a,const TMatrixT   <Element> &b);
    void MultT(const TMatrixTSym<Element> &a,const TMatrixTSym<Element> &b) { Mult(a,b); }
 
+   inline void SetElement(Int_t rown, Int_t coln, Element val);
+
    const Element *GetMatrixArray  () const override;
          Element *GetMatrixArray  () override;
    const Int_t   *GetRowIndexArray() const override { return nullptr; }
@@ -212,7 +215,6 @@ public:
    ClassDefOverride(TMatrixT,4) // Template of General Matrix class
 };
 
-#ifndef __CINT__
 // When building with -fmodules, it instantiates all pending instantiations,
 // instead of delaying them until the end of the translation unit.
 // We 'got away with' probably because the use and the definition of the
@@ -221,7 +223,6 @@ public:
 // In case we are building with -fmodules, we need to forward declare the
 // specialization in order to compile the dictionary G__Matrix.cxx.
 template <> TClass *TMatrixT<double>::Class();
-#endif // __CINT__
 
 
 template <class Element> inline const Element           *TMatrixT<Element>::GetMatrixArray() const { return fElements; }
@@ -285,6 +286,21 @@ template <class Element> inline Element &TMatrixT<Element>::operator()(Int_t row
    return (fElements[arown*this->fNcols+acoln]);
 }
 
+////////////////////////////////////////////////////////////////////////////////
+/// Efficiently sets element (rown,coln) equal to val
+/// Index bound checks can be deactivated by defining NDEBUG
+
+template <class Element>
+inline void TMatrixT<Element>::SetElement(Int_t rown, Int_t coln, Element val)
+{
+   assert(this->IsValid());
+   rown = rown - this->fRowLwb;
+   coln = coln - this->fColLwb;
+   assert((rown < this->fNrows && rown >= 0) && "SetElement() error: row index outside matrix range");
+   assert((coln < this->fNcols && coln >= 0) && "SetElement() error: column index outside matrix range");
+   fElements[rown * this->fNcols + coln] = val;
+}
+
 inline namespace TMatrixTAutoloadOps {
 
 template <class Element> TMatrixT<Element>  operator+  (const TMatrixT   <Element> &source1,const TMatrixT   <Element> &source2);
