@@ -165,9 +165,9 @@ public:
         Color_t color=-1, Short_t bordersize=-1, Short_t bordermode=-2);
    ~TPad() override;
    void              AbsCoordinates(Bool_t set) override { fAbsCoord = set; }
-   Double_t          AbsPixeltoX(Int_t px) override { return fAbsPixeltoXk + px*fPixeltoX; }
-   Double_t          AbsPixeltoY(Int_t py) override { return fAbsPixeltoYk + py*fPixeltoY; }
-   virtual void      AbsPixeltoXY(Int_t xpixel, Int_t ypixel, Double_t &x, Double_t &y);
+   Double_t          AbsPixeltoX(Double_t px) override { return fAbsPixeltoXk + px*fPixeltoX; }
+   Double_t          AbsPixeltoY(Double_t py) override { return fAbsPixeltoYk + py*fPixeltoY; }
+   void              AbsPixeltoXY(Double_t xpixel, Double_t ypixel, Double_t &x, Double_t &y) override;
    void              Add(TObject *obj, Option_t *opt = "", Bool_t modified = kTRUE) override;
    void              AddFirst(TObject *obj, Option_t *opt = "", Bool_t modified = kTRUE) override;
    void              AddExec(const char *name, const char *command) override;
@@ -186,6 +186,7 @@ public:
    void              CopyPixmaps() override;
    void              DeleteExec(const char *name) override;
    void              Divide(Int_t nx=1, Int_t ny=1, Float_t xmargin=0.01, Float_t ymargin=0.01, Int_t color=0) override; // *MENU*
+   void              DivideRatios(Int_t nrows, Int_t ncolumns, const std::vector<double>& widthRatios={}, const std::vector<double>& heightRatios={}, const double canvasTopMargin=0., const double canvasLeftMargin=0.);
    virtual void      DivideSquare(Int_t n, Float_t xmargin=0.01, Float_t ymargin=0.01, Int_t color=0);
    void              Draw(Option_t *option="") override;
    void              DrawClassObject(const TObject *obj, Option_t *option="") override;
@@ -300,15 +301,16 @@ public:
    void              PaintPolyLineNDC(Int_t n, Double_t *x, Double_t *y, Option_t *option="") override;
    void              PaintPolyMarker(Int_t n, Float_t *x, Float_t *y, Option_t *option="") override;
    void              PaintPolyMarker(Int_t n, Double_t *x, Double_t *y, Option_t *option="") override;
+   void              PaintMarker3D(Double_t x, Double_t y, Double_t z) override;
    void              PaintModified() override;
    void              PaintText(Double_t x, Double_t y, const char *text) override;
    void              PaintText(Double_t x, Double_t y, const wchar_t *text) override;
    void              PaintTextNDC(Double_t u, Double_t v, const char *text) override;
    void              PaintTextNDC(Double_t u, Double_t v, const wchar_t *text) override;
    virtual TPad     *Pick(Int_t px, Int_t py, TObjLink *&pickobj);
-   Double_t          PixeltoX(Int_t px) override;
-   Double_t          PixeltoY(Int_t py) override;
-   virtual void      PixeltoXY(Int_t xpixel, Int_t ypixel, Double_t &x, Double_t &y);
+   Double_t          PixeltoX(Double_t px) override;
+   Double_t          PixeltoY(Double_t py) override;
+   void              PixeltoXY(Double_t xpixel, Double_t ypixel, Double_t &x, Double_t &y) override;
    void              Pop() override;  // *MENU*
    void              Print(const char *filename="") const override;
    void              Print(const char *filename, Option_t *option) override;
@@ -381,8 +383,10 @@ public:
    Double_t          YtoPad(Double_t y) const override;
    Int_t             XtoPixel(Double_t x) const override;
    Int_t             YtoPixel(Double_t y) const override;
-   virtual void      XYtoAbsPixel(Double_t x, Double_t y, Int_t &xpixel, Int_t &ypixel) const;
-   virtual void      XYtoPixel(Double_t x, Double_t y, Int_t &xpixel, Int_t &ypixel) const;
+   void              XYtoAbsPixel(Double_t x, Double_t y, Int_t &xpixel, Int_t &ypixel) const override;
+   void              XYtoAbsPixel(Double_t x, Double_t y, Double_t &xpixel, Double_t &ypixel) const override;
+   void              XYtoPixel(Double_t x, Double_t y, Int_t &xpixel, Int_t &ypixel) const override;
+   void              XYtoPixel(Double_t x, Double_t y, Double_t &xpixel, Double_t &ypixel) const override;
 
    TObject          *CreateToolTip(const TBox *b, const char *text, Long_t delayms) override;
    void              DeleteToolTip(TObject *tip) override;
