@@ -69,6 +69,24 @@ sap.ui.define([
                if (this._logLevel >= 2)
                   console.log("GlViewerRCore.onInit - RenderCore.js loaded");
                RC = module;
+
+               RC.Canvas.prototype.generateCanvasDOM = function(id="eve7-rc-canvas") {
+                  if (RC.Canvas.prototype._xxcount === undefined) { RC.Canvas.prototype._xxcount = 0; }
+
+                  const canvasDOM = document.createElement("canvas");
+                  canvasDOM.id = id + "-" + RC.Canvas.prototype._xxcount;
+                  ++RC.Canvas.prototype._xxcount;
+
+                  //make it visually fill the positioned parent
+                  //set the display size of the canvas
+                  canvasDOM.style.width = "100%";
+                  canvasDOM.style.height = "100%";
+                  canvasDOM.style.padding = '0';
+                  canvasDOM.style.margin = '0';
+
+                  return canvasDOM;
+               };
+
                pthis.bootstrap();
             });
          } else {
@@ -127,7 +145,26 @@ sap.ui.define([
 
       createRCoreRenderer()
       {
-         this.canvas = new RC.Canvas(this.get_view().getDomRef());
+         let canvasParentDOM = document.createElement("div");
+         let vid = this.get_view().sId + "--rcore";
+         canvasParentDOM.setAttribute("id", vid);
+         canvasParentDOM.style.width = "100%";
+         canvasParentDOM.style.height = "100%";
+
+         // in case of openui5 rooter, the canvas element accumulates
+         // destroy the old canvas element
+	      let cn = this.get_view().getDomRef().childNodes;
+         let oldCanvas = -1;
+         for (let i =0; i < cn.length; ++i) {
+           if (cn[i].id === vid)
+                oldCanvas = i;
+         }
+         if (oldCanvas !== -1) {
+            this.get_view().getDomRef().removeChild(cn[oldCanvas]);
+         }
+
+         this.get_view().getDomRef().appendChild(canvasParentDOM);
+         this.canvas = new RC.Canvas(canvasParentDOM);
          let w = this.canvas.width;
          let h = this.canvas.height;
          this.fixCssSize();
@@ -277,7 +314,8 @@ sap.ui.define([
          this.ttip.setAttribute('class', 'eve_tooltip');
          this.ttip_text = document.createElement('div');
          this.ttip.appendChild(this.ttip_text);
-         this.get_view().getDomRef().appendChild(this.ttip);
+         this.canvas.parentDOM.appendChild(this.ttip)
+
 
          // Setup some event pre-handlers
          let glc = this;
@@ -339,7 +377,6 @@ sap.ui.define([
          });
 
          dome.addEventListener("mousedown", function(event) {
-            console.log("event mousedown", event);
             if (event.button == 0 || event.button == 2)
             {
                glc.handleOverlayMouseDown(event);
@@ -454,9 +491,14 @@ sap.ui.define([
          if (this._logLevel >= 2)
             console.log("GlViewerRenderCore.resetRenderer", sbbox, posV, negV, extV, extR);
 
+         let eveView = this.controller.mgr.GetElement(this.controller.eveViewerId);
+         let v1 = eveView.camera.V1;
+         let v2 = eveView.camera.V2;
+
+
          if (this.camera.isPerspectiveCamera)
          {
-            this.controls.setCamBaseMtx(new RC.Vector3(-1, 0, 0), new RC.Vector3(0, 1, 0)); //XOZ floor
+            this.controls.setCamBaseMtx(new RC.Vector3(v1[0], v1[1], v1[2]), new RC.Vector3(v2[0], v2[1], v2[2]));
             this.controls.screenSpacePanning = true;
 
             let lc = this.lights.children;
@@ -471,7 +513,7 @@ sap.ui.define([
          }
          else
          {
-            this.controls.setCamBaseMtx(new RC.Vector3(0, 0, 1), new RC.Vector3(0, 1, 0)); // XOY floor
+            this.controls.setCamBaseMtx(new RC.Vector3(v1[0], v1[1], v1[2]), new RC.Vector3(v2[0], v2[1], v2[2]));
             let ey = 1.02 * extV.y;
             let ex = ey / this.get_height() * this.get_width();
             this.camera._left   = -ex;
@@ -551,10 +593,6 @@ sap.ui.define([
          let bb = new RC.Box3();
          bb.setFromObject(this.scene);
 
-         console.log(formatFloat(bb.max.x), formatFloat(bb.min.x),
-         formatFloat(bb.max.y), formatFloat(bb.min.y),
-         formatFloat(bb.max.z), formatFloat(bb.min.z));
-
          let lines = [];
          lines.push({ "p": new RC.Vector3(bb.min.x, 0, 0), "c": new RC.Color(1, 0, 0), "text": "x " + formatFloat(bb.min.x) });
          lines.push({ "p": new RC.Vector3(bb.max.x, 0, 0), "c": new RC.Color(1, 0, 0), "text": "x " + formatFloat(bb.max.x) });
@@ -709,7 +747,7 @@ sap.ui.define([
 
          let state = this.rqt.pick(x, y, detect_depth);
 
-         console.log("pick state", state);
+         // console.log("pick state", state);
 
          if (state.object === null) {
             this.rqt.pick_end();
@@ -752,8 +790,6 @@ sap.ui.define([
 
          let state_overlay = this.rqt.pick_overlay(x, y, detect_depth);
 
-         console.log("Overlay pick state", state_overlay);
-
          if (state_overlay.object === null) {
             this.rqt.pick_end();
             return null;
@@ -822,7 +858,22 @@ sap.ui.define([
                console.log("GlViewerRCore onResizeTimeout -- canvas is not set yet.");
             return;
          }
+         {
+            let dome = this.get_view().getDomRef();
 
+            let vid = this.get_view().sId + "--rcore";
+            let hasCanvas = false;
+            for (const child of dome.children) {
+               if (child.id === vid) {
+                  hasCanvas = true;
+               }
+            }
+
+            if (hasCanvas === false)
+            {
+               dome.appendChild(this.canvas.parentDOM);
+            }
+         }
          this.floatCssSize();
          this.canvas.updateSize();
          let w = this.canvas.width;
@@ -879,7 +930,7 @@ sap.ui.define([
          let c = pstate.ctrl;
          let idx = c.extractIndex(pstate.instance);
 
-         c.elementHighlighted(idx, null, pstate.object);
+         c.elementHighlighted(idx, null, pstate.object)
 
          if (this.highlighted_top_object !== pstate.top_object)
          {
@@ -1029,7 +1080,7 @@ sap.ui.define([
 
       handleOverlayMouseUp()
       {
-         console.log("handleOverlayMouseUp");
+         // console.log("handleOverlayMouseUp");
          if(this.firstMouseDown == false)
          {
             this.firstMouseDown = true;
@@ -1048,7 +1099,7 @@ sap.ui.define([
 
       handleOverlayMouseDown(event)
       {
-         console.log("handleOverlayMouseDown");
+         // console.log("handleOverlayMouseDown");
          let x = event.offsetX * this.canvas.pixelRatio;
          let y = event.offsetY * this.canvas.pixelRatio;
          let overlay_pstate = this.render_for_Overlay_picking(x, y, false);
